"""Script for running a tsunami inundation scenario for Onslow, WA, Australia.

Source data such as elevation and boundary data is assumed to be available in
directories specified by project.py
The output sww file is stored in project.outputtimedir 

The scenario is defined by a triangular mesh created from project.polygon,
the elevation data and a simulated submarine landslide.

Ole Nielsen and Duncan Gray, GA - 2005 and Nick Bartzis, GA - 2006
"""


#-------------------------------------------------------------------------------# Import necessary modules
#-------------------------------------------------------------------------------

# Standard modules
import os
import time

# Related major packages
from pyvolution.shallow_water import Domain, Reflective_boundary, \
                            Dirichlet_boundary, Time_boundary, File_boundary
from pyvolution.data_manager import convert_dem_from_ascii2netcdf, dem2pts
from pyvolution.combine_pts import combine_rectangular_points_files 
from pyvolution.pmesh2domain import pmesh_to_domain_instance
#from geospatial_data import add_points_files

# Application specific imports
import project                 # Definition of file names and polygons
from smf import slump_tsunami  # Function for submarine mudslide 

from shutil import copy
from os import mkdir, access, F_OK

from geospatial_data import *
import sys
from pyvolution.util import Screen_Catcher

#-------------------------------------------------------------------------------
# Preparation of topographic data
#
# Convert ASC 2 DEM 2 PTS using source data and store result in source data
# Do for coarse and fine data
# Fine pts file to be clipped to area of interest
#-------------------------------------------------------------------------------

# filenames
#coarsedemname = project.coarsedemname

onshore_dem_name = project.onshore_dem_name
islands_dem_name = project.islands_dem_name
coast_points = project.coast_dem_name
offshore_points = project.offshore_dem_name

meshname = project.meshname+'.msh'

source_dir = project.boundarydir

# creates copy of code in output dir if dir doesn't exist
if access(project.outputtimedir,F_OK) == 0 :
    mkdir (project.outputtimedir)
copy (project.codedirname, project.outputtimedir + project.codename)
copy (project.codedir + 'run_onslow.py', project.outputtimedir + 'run_onslow.py')
print'output dir', project.outputtimedir

#normal screen output is stored in 
screen_output_name = project.outputtimedir + "screen_output.txt"
screen_error_name = project.outputtimedir + "screen_error.txt"

#used to catch screen output to file
sys.stdout = Screen_Catcher(screen_output_name)
#sys.stderr = Screen_Catcher(screen_output_name)
sys.stderr = Screen_Catcher(screen_error_name)


copied_files = False

# files to be used
files_used = [onshore_dem_name, offshore_points, coast_points,]

if sys.platform != 'win32':    
    copied_files = True
    for name in file_list:
        copy(name, )
'''    
#print' most file', project.MOST_dir + project.boundary_basename+'_ha.nc'
#if access(project.MOST_dir + project.boundary_basename+'_ha.nc',F_OK) == 1 :
#    print' most file', project.MOST_dir + project.boundary_basename

'''
# fine data (clipping the points file to smaller area)
# creates DEM from asc data
convert_dem_from_ascii2netcdf(onshore_dem_name, use_cache=True, verbose=True)

#creates pts file for onshore DEM
dem2pts(onshore_dem_name,
        easting_min=project.eastingmin,
        easting_max=project.eastingmax,
        northing_min=project.northingmin,
        northing_max= project.northingmax,
        use_cache=True, 
        verbose=True)

convert_dem_from_ascii2netcdf(islands_dem_name, use_cache=True, verbose=True)

#creates pts file for islands DEM
dem2pts(islands_dem_name, use_cache=True, verbose=True)

print'create G1'
G1 = Geospatial_data(file_name = project.offshore_dem_name + '.xya')

print'create G2'
G2 = Geospatial_data(file_name = project.onshore_dem_name + '.pts')

print'create G3'
G3 = Geospatial_data(file_name = project.coast_dem_name + '.xya')

print'create G4'
G4 = Geospatial_data(file_name = project.islands_dem_name + '.pts')

print'add G1+G2+G3+G4'
G = G1 + G2 + G3 + G4

print'export G'
G.export_points_file(project.combined_dem_name + '.pts')


#-------------------------------------------------------------------------------                                 
# Create the triangular mesh based on overall clipping polygon with a tagged
# boundary and interior regions defined in project.py along with
# resolutions (maximal area of per triangle) for each polygon
#-------------------------------------------------------------------------------

from pmesh.mesh_interface import create_mesh_from_regions
'''
# original
interior_res = 5000
high_res = 1500
interior_regions = [[project.poly_onslow, high_res],
                    [project.poly_thevenard, interior_res],
                    [project.poly_coast, interior_res]]
'''
#new
region_res = 50000
coast_res = 25000
onslow_res = 500
interior_regions = [[project.poly_onslow, onslow_res],
                    [project.poly_coast, coast_res],
                    [project.poly_region, region_res]]

print 'number of interior regions', len(interior_regions)

from caching import cache
_ = cache(create_mesh_from_regions,
          project.polyAll,
          {'boundary_tags': {'top': [0], 'topleft': [1],
                             'topleft1': [2], 'bottomleft': [3],
                             'bottom': [4], 'bottomright': [5],
                             'topright':[6]},
           'maximum_triangle_area': 250000,
           'filename': meshname,           
           'interior_regions': interior_regions},
          verbose = True, evaluate=True)


#-------------------------------------------------------------------------------                                 
# Setup computational domain
#-------------------------------------------------------------------------------                                 

domain = pmesh_to_domain_instance(meshname, Domain,
                                  use_cache = False,
                                  verbose = True)

print 'Number of triangles = ', len(domain)
print 'The extent is ', domain.get_extent()
print domain.statistics()

domain.set_name(project.basename)
domain.set_datadir(project.outputtimedir)
domain.set_quantities_to_be_stored(['stage', 'xmomentum', 'ymomentum'])

#-------------------------------------------------------------------------------
# Set up scenario (tsunami_source is a callable object used with set_quantity)
#-------------------------------------------------------------------------------
'''
tsunami_source = slump_tsunami(length=30000.0,
                               depth=400.0,
                               slope=6.0,
                               thickness=176.0, 
                               radius=3330,
                               dphi=0.23,
                               x0=project.slump_origin[0], 
                               y0=project.slump_origin[1], 
                               alpha=0.0, 
                               domain=domain)

'''
#-------------------------------------------------------------------------------                                 
# Setup initial conditions
#-------------------------------------------------------------------------------

tide = 0.0

domain.set_quantity('stage', tide)
domain.set_quantity('friction', 0.0) 
print 'hi and file',project.combined_dem_name + '.pts'

domain.set_quantity('elevation', 
#                    0.
#                    filename = project.onshore_dem_name + '.pts',
                    filename = project.combined_dem_name + '.pts',
#                    filename = project.offshore_dem_name + '.pts',
                    use_cache = True,
                    verbose = True,
                    alpha = 0.1
                    )

print 'hi1'

#-------------------------------------------------------------------------------                                 
# Setup boundary conditions (all reflective)
#-------------------------------------------------------------------------------
print 'start ferret2sww'
from pyvolution.data_manager import ferret2sww

south = project.south 
north = project.north
west = project.west
east = project.east

#note only need to do when an SWW file for the MOST boundary doesn't exist
cache(ferret2sww,
      (source_dir + project.boundary_basename,
       source_dir + project.boundary_basename), 
#      (project.MOST_dir + project.boundary_basename,
#       source_dir + project.boundary_basename), 
      {'verbose': True,
# note didn't work with the below
#       'minlat': south - 1,
#       'maxlat': north + 1,
#       'minlon': west - 1,
#       'maxlon': east + 1,
       'minlat': south,
       'maxlat': north,
       'minlon': west,
       'maxlon': east,
#       'origin': project.mesh_origin,
       'origin': domain.geo_reference.get_origin(),
       'mean_stage': tide,
       'zscale': 1,                 #Enhance tsunami
       'fail_on_NaN': False,
       'inverted_bathymetry': True},
      #evaluate = True,
       verbose = True, evaluate=True)


print 'Available boundary tags', domain.get_boundary_tags()

Bf = File_boundary(source_dir + project.boundary_basename + '.sww', 
                    domain, verbose = True)
Br = Reflective_boundary(domain)
Bd = Dirichlet_boundary([tide,0,0])


# 7 min square wave starting at 1 min, 6m high
Bw = Time_boundary(domain = domain,
                   f=lambda t: [(60<t<480)*6, 0, 0])

domain.set_boundary( {'top': Bf, 'topleft': Bf,
                             'topleft1': Bf, 'bottomleft': Bd,
                             'bottom': Br, 'bottomright': Br, 'topright': Bd} )

#-------------------------------------------------------------------------------                                 
# Evolve system through time
#-------------------------------------------------------------------------------
import time
t0 = time.time()

for t in domain.evolve(yieldstep = 450, finaltime = 10800): 
    domain.write_time()
    domain.write_boundary_statistics(tags = 'top')     

for t in domain.evolve(yieldstep = 1, finaltime = 17760
                       ,skip_initial_step = True): 
    domain.write_time()
    domain.write_boundary_statistics(tags = 'top')     

for t in domain.evolve(yieldstep = 480, finaltime = 36000
                       ,skip_initial_step = True): 
    domain.write_time()
    domain.write_boundary_statistics(tags = 'top')     
    
  
print 'That took %.2f seconds' %(time.time()-t0)

print 'finished'
